==> highlight-updates-0.0.1.tgz: package/src/model.ts <==
import { createCanvas, draw } from "./draw";

export interface UpdateRect {
  x: number;
  y: number;
  width: number;
  height: number;
  count: number;
  expirationTime: number;
}

export type UpdateRects = Map<HTMLElement, UpdateRect>;

export interface State {
  canvas: HTMLCanvasElement | null;
  updates: UpdateRects;
  timer: NodeJS.Timeout | null;
}

export interface HighlightOptions {
  colors?: string[];
  outlineColor?: string;
  displayDuration?: number;
  maxDisplayDuration?: number;
  container?: HTMLElement;
}

export function newHighlight({
  colors = [
    "#37afa9",
    "#63b19e",
    "#80b393",
    "#97b488",
    "#abb67d",
    "#beb771",
    "#cfb965",
    "#dfba57",
    "#efbb49",
    "#febc38",
  ],
  outlineColor = "#f0f0f0",
  displayDuration = 250,
  maxDisplayDuration = 3000,
  container = document.body,
}: HighlightOptions = {}) {
  const state: State = {
    updates: new Map(),
    timer: null,
    canvas: null,
  };

  function onResize() {
    const { canvas } = state;
    if (!canvas) return;
    canvas.width = window.innerWidth;
    canvas.height = window.innerHeight;
  }

  function destroy() {
    const { updates, timer, canvas } = state;

    window.removeEventListener("resize", onResize);

    updates.clear();
    if (timer !== null) setTimeout(timer as any);
    if (canvas !== null) canvas.remove();
  }

  function prepareDraw() {
    const nextDrawTime = draw(state, { colors, outlineColor });

    if (nextDrawTime > -1) {
      state.timer = setTimeout(prepareDraw, nextDrawTime);
    } else {
      destroy();
    }
  }

  return {
    update(dom: HTMLElement) {
      if (!state.canvas) {
        state.canvas = createCanvas(container);
        window.addEventListener("resize", onResize);
      }

      if (dom.nodeType === Node.TEXT_NODE) {
        if (dom.parentNode === null) return;
        dom = dom.parentNode as HTMLElement;
      }

      // Measure update
      const { updates } = state;
      const data = updates.get(dom);
      const rect = dom.getBoundingClientRect();

      const now = performance.now();
      const expirationTime = data
        ? Math.min(
            now + maxDisplayDuration,
            data.expirationTime + displayDuration
          )
        : now + displayDuration;

      updates.set(dom, {
        expirationTime,
        height: rect.height,
        width: rect.width,
        x: rect.x,
        y: rect.y,
        count: data ? data.count + 1 : 1,
      });

      prepareDraw();
    },
    destroy,
  };
}

==> highlight-updates-0.0.3.tgz: package/src/model.ts <==
import { createCanvas, draw } from "./draw";

export interface UpdateRect {
  x: number;
  y: number;
  width: number;
  height: number;
  count: number;
  expirationTime: number;
}

export type UpdateRects = Map<HTMLElement, UpdateRect>;

export interface State {
  canvas: HTMLCanvasElement | null;
  updates: UpdateRects;
  timer: NodeJS.Timeout | null;
}

export interface HighlightOptions {
  colors?: string[];
  outlineColor?: string;
  displayDuration?: number;
  maxDisplayDuration?: number;
  container?: HTMLElement;
}

export function newHighlight({
  colors = [
    "#37afa9",
    "#63b19e",
    "#80b393",
    "#97b488",
    "#abb67d",
    "#beb771",
    "#cfb965",
    "#dfba57",
    "#efbb49",
    "#febc38",
  ],
  outlineColor = "#f0f0f0",
  displayDuration = 250,
  maxDisplayDuration = 3000,
  container = document.body,
}: HighlightOptions = {}) {
  const state: State = {
    updates: new Map(),
    timer: null,
    canvas: null,
  };

  function onResize() {
    const { canvas } = state;
    if (!canvas) return;
    canvas.width = window.innerWidth;
    canvas.height = window.innerHeight;
  }

  function destroy() {
    const { updates, timer, canvas } = state;

    window.removeEventListener("resize", onResize);

    updates.clear();
    if (timer !== null) setTimeout(timer as any);
    if (canvas !== null) {
      canvas.remove();
      state.canvas = null;
    }
  }

  function prepareDraw() {
    const nextDrawTime = draw(state, { colors, outlineColor });

    if (nextDrawTime > -1) {
      state.timer = setTimeout(prepareDraw, nextDrawTime);
    } else {
      destroy();
    }
  }

  return {
    update(dom: HTMLElement) {
      if (!state.canvas) {
        state.canvas = createCanvas(container);
        window.addEventListener("resize", onResize);
        onResize();
      }

      if (dom.nodeType === Node.TEXT_NODE) {
        if (dom.parentNode === null) return;
        dom = dom.parentNode as HTMLElement;
      }

      // Measure update
      const { updates } = state;
      const data = updates.get(dom);
      const rect = dom.getBoundingClientRect();

      const now = performance.now();
      const expirationTime = data
        ? Math.min(
            now + maxDisplayDuration,
            data.expirationTime + displayDuration
          )
        : now + displayDuration;

      updates.set(dom, {
        expirationTime,
        height: rect.height,
        width: rect.width,
        x: rect.x,
        y: rect.y,
        count: data ? data.count + 1 : 1,
      });

      prepareDraw();
    },
    destroy,
  };
}
